/***************************************************************************
 *  Released into the public domain by Taj Morton 2007                     *
 *  tajmorton@gmail.com                                                    *
 *                                                                         *
 *  Do whatever you want to with this, but I'd appriciate it if you sent   *
 *  me the changes you make so I can merge them.                           *
 *                                                                         *
 *  Version: 1.0.1 2007-06-18 -- Add == and =(int) operators.              *
 *  Version 1.0:   2007-06-17                                              *
 ***************************************************************************/

#ifndef HAVE_DECIMAL_H
#define HAVE_DECIMAL_H
#include <string>

class decimal {
	public:
		enum RoundingMethod { Normal, AllUp, AllDown };
		decimal(const int precision, const int value = 0, RoundingMethod round = Normal);
		decimal(const int precision, const double value = 0.00, RoundingMethod round = Normal);
		~decimal() { };
		
		decimal & operator=(const double & rhs);
		
		decimal operator+(const decimal & rhs) const;
		decimal operator+(const int & rhs) const;
		decimal & operator+=(const decimal & rhs);
		decimal & operator+=(const int & rhs);
		
		decimal operator-(const decimal & rhs) const;
		decimal operator-(const int & rhs) const;
		decimal & operator-=(const decimal & rhs);
		decimal & operator-=(const int & rhs);
		
		decimal operator*(const decimal & rhs) const;
		decimal operator*(const int & rhs) const;
		decimal & operator*=(const decimal & rhs);
		decimal & operator*=(const int & rhs);
		
		decimal operator/(const decimal & rhs) const;
		decimal operator/(const int & rhs) const;
		decimal & operator/=(const decimal & rhs);
		decimal & operator/=(const int & rhs);
		
		bool operator>(const decimal & rhs) const;
		bool operator>(const int & rhs) const;
		
		bool operator<(const decimal & rhs) const;
		bool operator<(const int & rhs) const;
		
		bool operator==(const decimal & rhs) const;
		bool operator==(const int & rhs) const;
		
		void operator=(const int & rhs);
		
		decimal & operator++(); //prefix
		decimal operator++(int); //postfix
		
		void changePrecision(unsigned short int precision);
		
		friend std::ostream& operator<< (std::ostream&os, decimal d);
		
		// conversion helper functions:
		double to_double() const;
		operator std::string() const;
	
	private:
		bool m_is_positive;
		
		unsigned short int m_precision;
		unsigned long long int m_num;
		RoundingMethod m_round;
		
		std::string to_string() const;
		
		void setFromDouble(const int precision, const double value = 0.00);
		
		static void add(decimal & dec, const decimal & value);
		static void add(decimal & dec, const int & value);
		
		static void subtract(decimal & dec, const decimal & value);
		static void subtract(decimal & dec, const int & value);
		
		static void multiply(decimal & dec, const decimal & value);
		static void multiply(decimal & dec, const int & value);
		
		static void divide(decimal & dec, const decimal & value);
		static void divide(decimal & dec, const int & value);
};
#endif
